@@ -24,6 +24,10 @@
 #define DEFAULT_MAX_CATCH_UP_ROUNDS 10
 #define DEFAULT_MAX_CATCH_UP_ROUND_DURATION (5 * 1000)
 
+static int ioFsmCompat(struct raft *r,
+                       struct raft_io *io,
+                       struct raft_fsm *fsm);
+
 int raft_init(struct raft *r,
               struct raft_io *io,
               struct raft_fsm *fsm,
@@ -32,6 +36,12 @@ int raft_init(struct raft *r,
 {
     int rv;
     assert(r != NULL);
+
+    rv = ioFsmCompat(r, io, fsm);
+    if (rv != 0) {
+	goto err;
+    }
+
     r->io = io;
     r->io->data = r;
     r->fsm = fsm;
@@ -41,7 +51,7 @@ int raft_init(struct raft *r,
 
     r->id = id;
     /* Make a copy of the address */
-    r->address = HeapMalloc(strlen(address) + 1);
+    r->address = RaftHeapMalloc(strlen(address) + 1);
     if (r->address == NULL) {
         rv = RAFT_NOMEM;
         goto err;
@@ -78,7 +88,7 @@ int raft_init(struct raft *r,
     return 0;
 
 err_after_address_alloc:
-    HeapFree(r->address);
+    RaftHeapFree(r->address);
 err:
     assert(rv != 0);
     return rv;
@@ -227,3 +237,18 @@ unsigned long long raft_digest(const char *text, unsigned long long n)
 
     return byteFlip64(digest);
 }
+
+static int ioFsmCompat(struct raft *r,
+                       struct raft_io *io,
+                       struct raft_fsm *fsm)
+{
+    if ((fsm->version > 2 && fsm->snapshot_async != NULL)
+        && ((io->version < 2) || (io->async_work == NULL))) {
+        ErrMsgPrintf(r->errmsg,
+            "async snapshot requires io->version > 1 and async_work method.");
+        return -1;
+    }
+
+    return 0;
+}
+
