@@ -266,7 +266,7 @@ static int uvSnapshotLoadMeta(struct uv *uv,
         rv = RAFT_CORRUPT;
         goto err_after_open;
     }
-    buf.base = HeapMalloc(buf.len);
+    buf.base = RaftHeapMalloc(buf.len);
     if (buf.base == NULL) {
         rv = RAFT_NOMEM;
         goto err_after_open;
@@ -294,13 +294,13 @@ static int uvSnapshotLoadMeta(struct uv *uv,
         goto err_after_buf_malloc;
     }
 
-    HeapFree(buf.base);
+    RaftHeapFree(buf.base);
     UvOsClose(fd);
 
     return 0;
 
 err_after_buf_malloc:
-    HeapFree(buf.base);
+    RaftHeapFree(buf.base);
 
 err_after_open:
     close(fd);
@@ -338,11 +338,11 @@ static int uvSnapshotLoadData(struct uv *uv,
             tracef("decompress failed rv:%d", rv);
             goto err_after_read_file;
         }
-        HeapFree(buf.base);
+        RaftHeapFree(buf.base);
         buf = decompressed;
     }
 
-    snapshot->bufs = HeapMalloc(sizeof *snapshot->bufs);
+    snapshot->bufs = RaftHeapMalloc(sizeof *snapshot->bufs);
     snapshot->n_bufs = 1;
     if (snapshot->bufs == NULL) {
         rv = RAFT_NOMEM;
@@ -353,7 +353,7 @@ static int uvSnapshotLoadData(struct uv *uv,
     return 0;
 
 err_after_read_file:
-    HeapFree(buf.base);
+    RaftHeapFree(buf.base);
 err:
     assert(rv != 0);
     return rv;
@@ -468,10 +468,10 @@ static int uvRemoveOldSegmentsAndSnapshots(struct uv *uv,
 
 out:
     if (snapshots != NULL) {
-        HeapFree(snapshots);
+        RaftHeapFree(snapshots);
     }
     if (segments != NULL) {
-        HeapFree(segments);
+        RaftHeapFree(segments);
     }
     return rv;
 }
@@ -564,8 +564,8 @@ static void uvSnapshotPutFinish(struct uvSnapshotPut *put)
     int status = put->status;
     struct uv *uv = put->uv;
     assert(uv->snapshot_put_work.data == NULL);
-    HeapFree(put->meta.bufs[1].base);
-    HeapFree(put);
+    RaftHeapFree(put->meta.bufs[1].base);
+    RaftHeapFree(put);
     req->cb(req, status);
 }
 
@@ -573,14 +573,10 @@ static void uvSnapshotPutAfterWorkCb(uv_work_t *work, int status)
 {
     struct uvSnapshotPut *put = work->data;
     struct uv *uv = put->uv;
-    bool is_install = put->trailing == 0;
     assert(status == 0);
     uv->snapshot_put_work.data = NULL;
     uvSnapshotPutFinish(put);
-    if (is_install) {
-        UvUnblock(uv);
-    }
-    uvMaybeFireCloseCb(uv);
+    UvUnblock(uv);
 }
 
 /* Start processing the given put request. */
@@ -612,7 +608,7 @@ static void uvSnapshotPutBarrierCb(struct UvBarrier *barrier)
     }
 
     struct uv *uv = put->uv;
-    assert(put->trailing == 0);
+    assert((barrier->blocking && put->trailing == 0) || !barrier->blocking);
     put->barrier.data = NULL;
     /* If we're closing, abort the request. */
     if (uv->closing) {
@@ -635,14 +631,18 @@ int UvSnapshotPut(struct raft_io *io,
     void *cursor;
     unsigned crc;
     int rv;
+    raft_index next_index;
 
     uv = io->impl;
-    assert(!uv->closing);
+    if (uv->closing) {
+        return RAFT_CANCELED;
+    }
+
     assert(uv->snapshot_put_work.data == NULL);
 
     tracef("put snapshot at %lld, keeping %d", snapshot->index, trailing);
 
-    put = HeapMalloc(sizeof *put);
+    put = RaftHeapMalloc(sizeof *put);
     if (put == NULL) {
         rv = RAFT_NOMEM;
         goto err;
@@ -653,6 +653,7 @@ int UvSnapshotPut(struct raft_io *io,
     put->meta.timestamp = uv_now(uv->loop);
     put->trailing = trailing;
     put->barrier.data = put;
+    put->barrier.blocking = trailing == 0;
 
     req->cb = cb;
 
@@ -677,25 +678,25 @@ int UvSnapshotPut(struct raft_io *io,
     cursor = &put->meta.header[1];
     bytePut64(&cursor, crc);
 
-    /* If the trailing parameter is set to 0, it means that we're restoring a
+    /* - If the trailing parameter is set to 0, it means that we're restoring a
      * snapshot. Submit a barrier request setting the next append index to the
-     * snapshot's last index + 1. */
-    if (trailing == 0) {
-        rv = UvBarrier(uv, snapshot->index + 1, &put->barrier,
-                       uvSnapshotPutBarrierCb);
-        if (rv != 0) {
-            goto err_after_configuration_encode;
-        }
-    } else {
-        uvSnapshotPutStart(put);
+     * snapshot's last index + 1.
+     * - When we are only writing a snapshot during normal operation, we close
+     * all current open segments. New writes can continue on newly opened
+     * segments that will only contain entries that are newer than the snapshot,
+     * and we don't change append_next_index. */
+    next_index = (trailing == 0) ? (snapshot->index + 1) : uv->append_next_index;
+    rv = UvBarrier(uv, next_index, &put->barrier, uvSnapshotPutBarrierCb);
+    if (rv != 0) {
+        goto err_after_configuration_encode;
     }
 
     return 0;
 
 err_after_configuration_encode:
-    HeapFree(put->meta.bufs[1].base);
+    RaftHeapFree(put->meta.bufs[1].base);
 err_after_req_alloc:
-    HeapFree(put);
+    RaftHeapFree(put);
 err:
     assert(rv != 0);
     return rv;
@@ -723,10 +724,10 @@ static void uvSnapshotGetWorkCb(uv_work_t *work)
         if (rv != 0) {
             get->status = rv;
         }
-        HeapFree(snapshots);
+        RaftHeapFree(snapshots);
     }
     if (segments != NULL) {
-        HeapFree(segments);
+        RaftHeapFree(segments);
     }
 out:
     return;
@@ -741,7 +742,7 @@ static void uvSnapshotGetAfterWorkCb(uv_work_t *work, int status)
     struct uv *uv = get->uv;
     assert(status == 0);
     QUEUE_REMOVE(&get->queue);
-    HeapFree(get);
+    RaftHeapFree(get);
     req->cb(req, snapshot, req_status);
     uvMaybeFireCloseCb(uv);
 }
@@ -757,7 +758,7 @@ int UvSnapshotGet(struct raft_io *io,
     uv = io->impl;
     assert(!uv->closing);
 
-    get = HeapMalloc(sizeof *get);
+    get = RaftHeapMalloc(sizeof *get);
     if (get == NULL) {
         rv = RAFT_NOMEM;
         goto err;
@@ -766,7 +767,7 @@ int UvSnapshotGet(struct raft_io *io,
     get->req = req;
     req->cb = cb;
 
-    get->snapshot = HeapMalloc(sizeof *get->snapshot);
+    get->snapshot = RaftHeapMalloc(sizeof *get->snapshot);
     if (get->snapshot == NULL) {
         rv = RAFT_NOMEM;
         goto err_after_req_alloc;
@@ -786,9 +787,9 @@ int UvSnapshotGet(struct raft_io *io,
     return 0;
 
 err_after_snapshot_alloc:
-    HeapFree(get->snapshot);
+    RaftHeapFree(get->snapshot);
 err_after_req_alloc:
-    HeapFree(get);
+    RaftHeapFree(get);
 err:
     assert(rv != 0);
     return rv;
