@@ -39,7 +39,7 @@ static void uvTruncateWorkCb(uv_work_t *work)
         goto err;
     }
     if (snapshots != NULL) {
-        HeapFree(snapshots);
+        RaftHeapFree(snapshots);
     }
     assert(segments != NULL);
 
@@ -86,13 +86,13 @@ static void uvTruncateWorkCb(uv_work_t *work)
         goto err_after_list;
     }
 
-    HeapFree(segments);
+    RaftHeapFree(segments);
     truncate->status = 0;
 
     return;
 
 err_after_list:
-    HeapFree(segments);
+    RaftHeapFree(segments);
 err:
     assert(rv != 0);
     truncate->status = rv;
@@ -107,7 +107,7 @@ static void uvTruncateAfterWorkCb(uv_work_t *work, int status)
         uv->errored = true;
     }
     uv->truncate_work.data = NULL;
-    HeapFree(truncate);
+    RaftHeapFree(truncate);
     UvUnblock(uv);
 }
 
@@ -119,7 +119,7 @@ static void uvTruncateBarrierCb(struct UvBarrier *barrier)
 
     /* If we're closing, don't perform truncation at all and abort here. */
     if (uv->closing) {
-        HeapFree(truncate);
+        RaftHeapFree(truncate);
         return;
     }
 
@@ -152,7 +152,7 @@ int UvTruncate(struct raft_io *io, raft_index index)
     assert(index > 0);
     assert(index < uv->append_next_index);
 
-    truncate = HeapMalloc(sizeof *truncate);
+    truncate = RaftHeapMalloc(sizeof *truncate);
     if (truncate == NULL) {
         rv = RAFT_NOMEM;
         goto err;
@@ -160,6 +160,7 @@ int UvTruncate(struct raft_io *io, raft_index index)
     truncate->uv = uv;
     truncate->index = index;
     truncate->barrier.data = truncate;
+    truncate->barrier.blocking = true;
 
     /* Make sure that we wait for any inflight writes to finish and then close
      * the current segment. */
@@ -171,7 +172,7 @@ int UvTruncate(struct raft_io *io, raft_index index)
     return 0;
 
 err_after_req_alloc:
-    HeapFree(truncate);
+    RaftHeapFree(truncate);
 err:
     assert(rv != 0);
     return rv;
