@@ -24,7 +24,8 @@
     do {                                                                       \
         unsigned _n = DEFAULT_N;                                               \
         bool _pre_vote = false;                                                \
-        int _fsm_version = 2;                                                  \
+        bool _ss_async = false;                                                \
+        int _fsm_version = 3;                                                  \
         unsigned _hb = 0;                                                      \
         unsigned _i;                                                           \
         int _rv;                                                               \
@@ -39,6 +40,10 @@
             _hb =                                                              \
                 atoi(munit_parameters_get(params, CLUSTER_HEARTBEAT_PARAM));   \
         }                                                                      \
+        if (munit_parameters_get(params, CLUSTER_SS_ASYNC_PARAM) != NULL) {    \
+            _ss_async =                                                        \
+                atoi(munit_parameters_get(params, CLUSTER_SS_ASYNC_PARAM));    \
+        }                                                                      \
         if (munit_parameters_get(params, CLUSTER_FSM_VERSION_PARAM) != NULL) { \
             _fsm_version =                                                     \
                 atoi(munit_parameters_get(params, CLUSTER_FSM_VERSION_PARAM)); \
@@ -47,9 +52,13 @@
         _rv = raft_fixture_initialize(&f->cluster);                            \
         munit_assert_int(_rv, ==, 0);                                          \
         for (_i = 0; _i < _n; _i++) {                                          \
-            FsmInit(&f->fsms[_i], _fsm_version);                               \
             _rv = raft_fixture_grow(&f->cluster, &f->fsms[_i]);                \
             munit_assert_int(_rv, ==, 0);                                      \
+            if (!_ss_async || _fsm_version < 3) {                              \
+                FsmInit(&f->fsms[_i], _fsm_version);                           \
+            } else {                                                           \
+                FsmInitAsync(&f->fsms[_i], _fsm_version);                      \
+            }                                                                  \
         }                                                                      \
         for (_i = 0; _i < _n; _i++) {                                          \
             raft_set_pre_vote(raft_fixture_get(&f->cluster, _i), _pre_vote);   \
@@ -83,6 +92,9 @@
 #define CLUSTER_HEARTBEAT_PARAM "cluster-heartbeat"
 
 /* Munit parameter for setting snapshot behaviour */
+#define CLUSTER_SS_ASYNC_PARAM "cluster-snapshot-async"
+
+/* Munit parameter for setting fsm version */
 #define CLUSTER_FSM_VERSION_PARAM "fsm-version"
 
 /* Get the number of servers in the cluster. */
